import java.util.Arrays
import scala.actors.Futures._

object EratosthenesPar {
  def sieve(end : Int, threads : Int) : Array[Boolean] = {
    val sieve = new Array[Boolean](end + 1)

    for (i <- 2 to Math.sqrt(end).toInt) {
      if (!sieve(i)) {
        val c = i*i
        val perThread = (end-c+1) / (threads*i)
        val tasks = for (t <- 0 until threads) yield future {
          val begin = c + t * i * perThread
          val end2 = if (t == threads -1) end else c + (t+1) * i * perThread
          for (x <- begin to end2 by i)
            sieve(x) = true
        }

        awaitAll(20000L, tasks : _*)
      }
    }

    sieve
  }

  def main(args : Array[String]) {
    val target = 100000000

    for (threads <- List(1,2,4)) {
      val startTime = System.currentTimeMillis()

      sieve(target, threads)

      val endTime = System.currentTimeMillis()
      println(threads + " thread duration for interval [2, " + target + "] is "
        + (endTime - startTime) + " ms")
    }
  }
}
